UNIT mTSender;

interface

uses
  mTNetzHW, classes, windows, uUtil;

type
   TEreignis = procedure of object;

   TSender = CLASS(TThread)

     // weitere Attribute
     private
        rahmen   : array [ 0..20 ] of boolean; // Bits als boolean
        crc      : byte;  // 4-Bit-CRC
        dat      : byte;  // 8-Bit-Daten
        eadr     : byte;  // 4-Bit-eigene Adresse
        zadr     : byte;  // 4-Bit-Zieladresse
        baud     : integer;
        sendeStr : string;
        HW       : TNetzHW;
     public
        OnStart,OnEnd : TEreignis;
        
     // weitere Methoden
     public
        constructor create; virtual;
        procedure execute; override;
        procedure SetBaud (pBaud: integer); virtual;
        procedure SetCrc (pCrc: byte); virtual;
        procedure SetDat (pDat: byte); virtual;
        procedure SetEadr (pEadr: byte); virtual;
        procedure SetSendeStr (pSendeStr: string); virtual;
        procedure SetZadr (pZadr: byte); virtual;
        procedure SetzeHW (pHW: TNetzHW); virtual;
        procedure berechneCRC;
        procedure verpacke_Rahmen; // nicht berschreibbar
        function GetBaud : integer; virtual;
        function GetCrc : byte; virtual;
        function GetDat : byte; virtual;
        function GetEadr : byte; virtual;
        function GetZadr : byte; virtual;
   end;

implementation

//+---------------------------------------------------------------------
//|         TSender: Methodendefinition
//+---------------------------------------------------------------------

//-------- create (public) ---------------------------------------------
constructor TSender.create;
begin
  inherited create(true);
  FreeOnTerminate := true;
  zadr := 1;
  eadr := 7;
  crc := 7;
end;

//-------- execute (public) --------------------------------------------
procedure TSender.execute;
var
  n,i,j    : integer;
  b        : byte;
  bitZeit  : integer;
begin
  // Start anzeigen
  if Assigned(OnStart) then synchronize(OnStart);
  // Bitzeit berechnen
  bitZeit := round(1000/baud);
  n := Length(SendeStr);
  i := 1;
  while not Terminated and (i <= n) do
  begin
    dat := ord(sendeStr[i]);
    berechneCRC;
    verpacke_Rahmen;
    // StartBit
    HW.SetAus(true);
    sleep(bitZeit);
    // RahmenBits
    for j := 1 to 20 do
    begin
      HW.SetAus(rahmen[j]);
      sleep(bitZeit);
    end;
    // 3 StoppBits
    HW.SetAus(false);
    sleep(3*bitZeit);
    // nchstes Bit
    Inc(i);
  end;
  // Ende anzeigen
  if Assigned(OnEnd) then synchronize(OnEnd);
end;

//-------- SetBaud (public) --------------------------------------------
procedure TSender.SetBaud (pBaud: integer);
begin
  baud := pBaud;
end;

//-------- SetCrc (public) ---------------------------------------------
procedure TSender.SetCrc (pCrc: byte);
begin
  crc := pCrc;
end;

//-------- SetDat (public) ---------------------------------------------
procedure TSender.SetDat (pDat: byte);
begin
  dat := pDat;
end;

//-------- SetEadr (public) --------------------------------------------
procedure TSender.SetEadr (pEadr: byte);
begin
  eadr := pEadr
end;

//-------- SetSendeStr (public) ----------------------------------------
procedure TSender.SetSendeStr (pSendeStr: string);
begin
  sendeStr := pSendeStr;
end;

//-------- SetZadr (public) --------------------------------------------
procedure TSender.SetZadr (pZadr: byte);
begin
  zadr := pZadr;
end;

//-------- SetzeHW (public) --------------------------------------------
procedure TSender.SetzeHW (pHW: TNetzHW);
begin
  HW := pHW;
end;

procedure TSender.berechneCRC;
var
  nachricht : word;
begin
  nachricht := zadr; nachricht := nachricht shl 4;
  nachricht := nachricht+eadr; nachricht := nachricht shl 8;
  nachricht := nachricht+dat;
  // Form1.mSend.Lines.Add(IntToStr(nachricht)); DEBUG
  crc := crc4(nachricht);
end;

//-------- verpacke_Rahmen (public) ------------------------------------
procedure TSender.verpacke_Rahmen;
var
  i     : integer;
  h1,h2,h3 : byte;
begin
  h1 := zadr; h2 := eadr; h3 := crc;
  for i := 4 downto 1 do
  begin
    rahmen[i] := (h1 mod 2 = 1); h1 := h1 div 2;
    rahmen[4+i] := (h2 mod 2 = 1); h2 := h2 div 2;
    rahmen[16+i] := (h3 mod 2 = 1); h3 := h3 div 2;
  end;
  h1 := dat;
  for i := 8 downto 1 do
  begin
    rahmen[8+i] := (h1 mod 2 = 1); h1 := h1 div 2;
  end;
end;

//-------- GetBaud (public) --------------------------------------------
function TSender.GetBaud : integer;
begin
  result := baud;
end;

//-------- GetCrc (public) ---------------------------------------------
function TSender.GetCrc : byte;
begin
  result := crc;
end;

//-------- GetDat (public) ---------------------------------------------
function TSender.GetDat : byte;
begin
  result := dat;
end;

//-------- GetEadr (public) --------------------------------------------
function TSender.GetEadr : byte;
begin
  result := eadr;
end;

//-------- GetZadr (public) --------------------------------------------
function TSender.GetZadr : byte;
begin
  result := zadr;
end;

end.
